﻿using EA;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using EmfAddin.Utils;

namespace EmfAddin.Edit.UI
{
    public partial class EPackagePropertiesControl : UserControl
    {
        public event EventHandler<EventArgs> ContentChanged;

        private Package package;
        public Package Package
        { 
            get
            {
                return package;
            }
            
            set
            {
                this.package = null;
                if (value != null)
                {
                    nameTextBox.Enabled = true;
                    nsPrefixTextBox.Enabled = true;
                    nsUriTextBox.Enabled = true;

                    nameTextBox.Text = value.Name;
                    nsPrefixTextBox.Text = value.Element.GetFirstTaggedValueValue(EMFUtils.TAGGED_VALUE_E_PACKAGE_NS_PREFIX);
                    nsUriTextBox.Text = value.Element.GetFirstTaggedValueValue(EMFUtils.TAGGED_VALUE_E_PACKAGE_NS_URI);
                    this.package = value;
                }
                else
                {
                    nameTextBox.Enabled = false;
                    nsPrefixTextBox.Enabled = false;
                    nsUriTextBox.Enabled = false;
                    nameTextBox.Text = "";
                    nsPrefixTextBox.Text = "";
                    nsUriTextBox.Text = "";
                }
            } 
        }

        public EPackagePropertiesControl()
        {
            InitializeComponent();
        }

        private void NameTextChanged(object sender, EventArgs e)
        {
            OnContentChanged(e);
        }

        private void NsPrefixTextChanged(object sender, EventArgs e)
        {
            OnContentChanged(e);
        }

        private void NsUriTextChanged(object sender, EventArgs e)
        {
            OnContentChanged(e);
        }

        private void OnContentChanged(EventArgs e)
        {
            EventHandler<EventArgs> handler = ContentChanged;
            if (handler != null)
            {
                handler(this, e);
            }
        }

        public void UpdatePackage()
        {
            if (package != null)
            {
                bool changed = false;
                if (!string.Equals(package.Element.GetFirstTaggedValueValue("NS_Prefix"), nsPrefixTextBox.Text))
                {
                    if (package.Element.SetFirstTaggedValueValue("NS_Prefix", nsPrefixTextBox.Text))
                    {
                        changed = true;
                    }
                    else
                    {
                        MessageBox.Show(this, "Failed to update tagged value 'NS_Prefix' of package '" + package.Name + "' with ID " + package.PackageID + ".", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
                if (!string.Equals(package.Element.GetFirstTaggedValueValue("NS_URI"), nsUriTextBox.Text))
                {
                    if (package.Element.SetFirstTaggedValueValue("NS_URI", nsUriTextBox.Text))
                    {
                        changed = true;
                    }
                    else
                    {
                        MessageBox.Show(this, "Failed to update tagged value 'NS_URI' of package '" + package.Name + "' with ID " + package.PackageID + ".", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
                if (!string.Equals(package.Name, nameTextBox.Text))
                {
                    package.Name = nameTextBox.Text;
                    changed = true;
                }
                if (changed && !package.Update())
                {
                    MessageBox.Show(this, "Failed to update name of package '" + package.Name + "' with ID " + package.PackageID + ".", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }
    }
}
