﻿using EA;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using EmfAddin.Utils;

namespace EmfAddin.Edit.UI
{
    [ComVisible(true)]
    public class EcoreEditControl : UserControl
    {
        public const string MESSAGE_ELEMENT_NOT_SUPPORTED = "The selected element is not supported.";

        private System.ComponentModel.IContainer components = null;

        private Control shownControl;

        private bool ignoreContextChanges = false;

        private bool changeIgnored = false;

        public EcoreEditControl()
        {
            InitializeComponent();
        }

        private void InitializeComponent()
        {
            this.SuspendLayout();
            // 
            // shownControl
            // 
            ShowNotSupportedMessage();
            // 
            // EcoreEditControl
            // 
            this.AutoScaleDimensions = new System.Drawing.SizeF(6F, 13F);
            this.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Font;
            this.Margin = new System.Windows.Forms.Padding(2);
            this.Name = "EcoreEditControl";
            this.Size = new System.Drawing.Size(300, 200);
            this.ResumeLayout(false);

        }

        protected override void Dispose(bool disposing)
        {
            if (disposing && (components != null))
            {
                components.Dispose();
            }
            base.Dispose(disposing);
        }

        public void Show(Repository repository, string guid, ObjectType ot)
        {
            if (ignoreContextChanges)
            {
                changeIgnored = true;
            }
            else if (guid == null || guid.Length == 0)
            {
                ShowNotSupportedMessage();
            }
            else if (ObjectType.otPackage == ot)
            {
                Package package = repository.GetPackageByGuid(guid);
                if (package.HasStereotype(EMFUtils.STEREOTYPE_E_PACKAGE))
                {
                    ShowPackage(repository, package);
                }
                else
                {
                    ShowNotSupportedMessage();
                }
            }
            else
            {
                ShowNotSupportedMessage();
            }
        }

        protected void ShowNotSupportedMessage()
        {
            Label label = new Label();
            label.Text = MESSAGE_ELEMENT_NOT_SUPPORTED;
            ShowControl(label);
        }

        protected void ShowPackage(Repository repository, Package package)
        {
            EPackagePropertiesControl control = new EPackagePropertiesControl();
            control.ContentChanged += (object sender, EventArgs e) => 
            {
                try
                {
                    ignoreContextChanges = true;
                    changeIgnored = false;
                    control.UpdatePackage();
                }
                finally
                {
                    if (changeIgnored)
                    {
                        repository.ShowInProjectView(package);
                    }
                    repository.RefreshOpenDiagrams(false);
                    ignoreContextChanges = false;
                }
            };
            control.Package = package;
            ShowControl(control);
        }

        protected void ShowControl(Control control)
        {
            this.SuspendLayout();
            DisposeShownControl();
            this.shownControl = control;
            this.shownControl.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
           | System.Windows.Forms.AnchorStyles.Left)
           | System.Windows.Forms.AnchorStyles.Right)));
            this.shownControl.Location = new System.Drawing.Point(5, 5);
            this.shownControl.Margin = new System.Windows.Forms.Padding(2);
            this.shownControl.Name = "packageControl";
            this.shownControl.Size = new System.Drawing.Size(this.Size.Width - 10, this.Size.Height - 10);
            this.shownControl.TabIndex = 0;
            this.Controls.Add(this.shownControl);
            this.ResumeLayout(true);
        }

        protected void DisposeShownControl()
        {
            if (shownControl != null)
            {
                Controls.Remove(shownControl);
                shownControl.Dispose();
            }
        }
    }
}
