﻿using EA;
using EmfAddin.Utils;
using System;
using System.Collections.Generic;
using System.Xml.Linq;

namespace EmfAddin.IntegrityChecks.Core
{
    public class EClassStereotypeIntegrityCheck
    {
        private readonly Repository repository;

        public EClassStereotypeIntegrityCheck(Repository repository)
        {
            this.repository = repository;
        }

        public IList<EClassStereotypeProblem> Check()
        {
            IList<EClassStereotypeProblem> problems = new List<EClassStereotypeProblem>();
            string sql = "SELECT t.Object_ID, t.Object_Type, t.Name " +
                         "FROM t_object t INNER JOIN t_xref x ON t.ea_guid = x.Client " +
                         "WHERE t.Object_Type NOT IN ('Class', 'Interface') AND x.Name = 'Stereotypes' AND x.Type = 'element property' AND x.Description LIKE '*Name=EClass;FQName=Ecore Profile*'";
            foreach (XElement element in repository.SQLQuery2(sql))
            {
                int objectId = element.GetChildElementValueAsInt("Object_ID");
                string name = element.GetChildElementValue("Name");
                string type = element.GetChildElementValue("Object_Type");
                problems.Add(new EClassStereotypeProblem(objectId, name, type));
            }
            return problems;
        }

        public void Resolve(IList<EClassStereotypeProblem> problems)
        {
            foreach (EClassStereotypeProblem problem in problems)
            {
                Resolve(problem);
            }
        }

        private void Resolve(EClassStereotypeProblem problems)
        {
            Element element = repository.GetElementByID(problems.ElementID);
            if (element != null)
            {
                IList<string> stereotypes = EAUtils.ParseStereotypes(element.StereotypeEx);
                if (stereotypes.Remove(EMFUtils.STEREOTYPE_E_CLASS))
                {
                    element.StereotypeEx = string.Join(",", stereotypes);
                    if (!element.Update())
                    {
                        throw new Exception("Failed to save " + element.Type + " '" + element.Name + "'.");
                    }
                }
            }
        }
    }
}
