﻿using EA;
using EmfAddin.Utils;
using System;
using System.Collections.Generic;
using System.Xml.Linq;

namespace EmfAddin.IntegrityChecks.Core
{
    public class EPackageTaggedValuesIntegerityCheck
    {
        private readonly Repository repository;

        public EPackageTaggedValuesIntegerityCheck(Repository repository)
        {
            this.repository = repository;
        }

        public IList<EPackageTaggedValueProblem> Check()
        {
            IList<EPackageTaggedValueProblem> problems = new List<EPackageTaggedValueProblem>();
            string sql = "SELECT p.Package_ID, p.Name, t.Property " +
                         "FROM ((t_package p INNER JOIN t_object o ON p.ea_guid = o.ea_guid) INNER JOIN t_xref x ON p.ea_guid = x.Client) INNER JOIN t_objectproperties t ON t.Object_ID = o.Object_ID " +
                         "WHERE x.Description LIKE '*" + EMFUtils.ECORE_PROFILE + EAUtils.STEREOTYPE_SEPARATOR + EMFUtils.STEREOTYPE_E_PACKAGE + "*' " +
                         "ORDER BY p.Package_ID";
            int lastPackageID = 0;
            string lastName = null;
            ISet<string> foundTaggedValues = null;
            foreach (XElement element in repository.SQLQuery2(sql))
            {
                int packageID = element.GetChildElementValueAsInt("Package_ID");
                string name = element.GetChildElementValue("Name");
                string property = element.GetChildElementValue("Property");
                if (lastPackageID != packageID)
                {
                    if (lastPackageID != 0)
                    {
                        if (!foundTaggedValues.Contains(EMFUtils.TAGGED_VALUE_E_PACKAGE_NS_PREFIX))
                        {
                            problems.Add(new EPackageTaggedValueProblem(lastPackageID, lastName, EMFUtils.TAGGED_VALUE_E_PACKAGE_NS_PREFIX));
                        }
                        if (!foundTaggedValues.Contains(EMFUtils.TAGGED_VALUE_E_PACKAGE_NS_URI))
                        {
                            problems.Add(new EPackageTaggedValueProblem(lastPackageID, lastName, EMFUtils.TAGGED_VALUE_E_PACKAGE_NS_URI));
                        }
                    }
                    lastPackageID = packageID;
                    lastName = name;
                    foundTaggedValues = new HashSet<string>();
                }
                foundTaggedValues.Add(property);
            }
            if (!foundTaggedValues.Contains(EMFUtils.TAGGED_VALUE_E_PACKAGE_NS_PREFIX))
            {
                problems.Add(new EPackageTaggedValueProblem(lastPackageID, lastName, EMFUtils.TAGGED_VALUE_E_PACKAGE_NS_PREFIX));
            }
            if (!foundTaggedValues.Contains(EMFUtils.TAGGED_VALUE_E_PACKAGE_NS_URI))
            {
                problems.Add(new EPackageTaggedValueProblem(lastPackageID, lastName, EMFUtils.TAGGED_VALUE_E_PACKAGE_NS_URI));
            }
            return problems;
        }

        public void Resolve()
        {
            repository.SynchProfile(EMFUtils.ECORE_PROFILE, EMFUtils.STEREOTYPE_E_PACKAGE);
        }
    }
}
