﻿using EA;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Linq;

namespace EmfAddin.Utils
{
    public static class EAUtils
    {
        public const string STEREOTYPE_SEPARATOR = "::";

        public static IEnumerable<XElement> SQLQuery2(this Repository repository, string sqlQuery)
        {
            string xmlData = repository.SQLQuery(sqlQuery);
            XDocument document = XDocument.Parse(xmlData);
            return document.GetEARows();
        }

        public static IEnumerable<XElement> GetEARows(this XDocument document)
        {
            return document.Descendants("Row");
        }

        public static string GetChildElementValue(this XElement parent, string childName)
        {
            return GetChildElementValue(parent, childName, string.Empty);
        }

        public static string GetChildElementValue(this XElement parent, string childName, string defaultValue)
        {
            if (parent != null)
            {
                var childElement = parent.Element(childName);
                return childElement != null ? childElement.Value : defaultValue;
            }
            else
            {
                return defaultValue;
            }
        }

        public static int GetChildElementValueAsInt(this XElement parent, string childName)
        {
            var value = GetChildElementValue(parent, childName);
            int operationId;
            return int.TryParse(value, out operationId) ? operationId : 0;
        }

        public static IList<string> ParseStereotypes(string stereotypeEx)
        {
            if (stereotypeEx != null && stereotypeEx.Length >= 1)
            {
                return stereotypeEx.Split(',').ToList<string>();
            }
            else
            {
                return new List<string>();
            }
        }

        public static bool SetFirstTaggedValueValue(this Element element, string name, string value)
        {
            if (element != null)
            {
                TaggedValue taggedValue = element.GetFirstTaggedValue(name);
                if (taggedValue == null)
                {
                    taggedValue = element.TaggedValues.AddNew(name, null);
                }
                taggedValue.Value = value;
                return taggedValue.Update();
            }
            else
            {
                return false;
            }
        }

        public static string GetFirstTaggedValueValue(this Element element, string name)
        {
            TaggedValue value = element.GetFirstTaggedValue(name);
            return value != null ? value.Value : null;
        }

        public static TaggedValue GetFirstTaggedValue(this Element element, string name)
        {
            if (element != null)
            {
                element.TaggedValues.Refresh();
                foreach (TaggedValue value in element.TaggedValues)
                {
                    if (string.Equals(name, value.Name))
                    {
                        return value;
                    }
                }
            }
            return null;
        }

        public static bool HasStereotype(this Package package, string stereotype)
        {
            if (package != null)
            {
                return package.StereotypeEx.Split(',').Any(s => s == stereotype);
            }
            return false;
        }
    }
}
